local singleplayer = minetest.is_singleplayer()
local setting = minetest.setting_getbool("enable_tnt")
if (not singleplayer and setting ~= true) or
		(singleplayer and setting == false) then
	return
end
local loss_prob = {}
loss_prob["default:cobble"] = 3
loss_prob["default:dirt"] = 4
local radius = tonumber(minetest.setting_get("tnt_radius") or 9)

local cid_data = {}
minetest.after(0, function()
	for name, def in pairs(minetest.registered_nodes) do
		cid_data[minetest.get_content_id(name)] = {
			name = name,
			drops = def.drops,
			flammable = def.groups.flammable,
		}
	end
end)

local function rand_pos(center, pos, radius)
	pos.x = center.x + math.random(-radius, radius)
	pos.z = center.z + math.random(-radius, radius)
end

local function eject_drops(drops, pos, radius)
	local drop_pos = vector.new(pos)
	for _, item in pairs(drops) do
		local count = item:get_count()
		local max = item:get_stack_max()
		if count > max then
			item:set_count(max)
		end
		while count > 0 do
			if count < max then
				item:set_count(count)
			end
			rand_pos(pos, drop_pos, radius)
			local obj = minetest.add_item(drop_pos, item)
			if obj then
				obj:get_luaentity().collect = true
				obj:setacceleration({x=0, y=-10, z=0})
				obj:setvelocity({x=math.random(-3, 3), y=10,
						z=math.random(-3, 3)})
			end
			count = count - max
		end
	end
end

local function add_drop(drops, item)
	item = ItemStack(item)
	local name = item:get_name()
	if loss_prob[name] ~= nil and math.random(1, loss_prob[name]) == 1 then
		return
	end

	local drop = drops[name]
	if drop == nil then
		drops[name] = item
	else
		drop:set_count(drop:get_count() + item:get_count())
	end
end

local fire_node = {name="fire:basic_flame"}

local function destroy(drops, pos, cid)
	if minetest.is_protected(pos, "") then
		return
	end
	local def = cid_data[cid]
	if def and def.flammable then
		minetest.set_node(pos, fire_node)
	else
		minetest.remove_node(pos)
		if def then
			local node_drops = minetest.get_node_drops(def.name, "")
			for _, item in ipairs(node_drops) do
				add_drop(drops, item)
			end
		end
	end
end

local function calc_velocity(pos1, pos2, old_vel, power)
	local vel = vector.direction(pos1, pos2)
	vel = vector.normalize(vel)
	vel = vector.multiply(vel, power)

	-- Divide by distance
	local dist = vector.distance(pos1, pos2)
	dist = math.max(dist, 1)
	vel = vector.divide(vel, dist)

	-- Add old velocity
	vel = vector.add(vel, old_vel)
	return vel
end

local function entity_physics(pos, radius)
	radius = radius * 2
	local objs = minetest.get_objects_inside_radius(pos, radius)
	for _, obj in pairs(objs) do
		local obj_pos = obj:getpos()
		local obj_vel = obj:getvelocity()
		local dist = math.max(1, vector.distance(pos, obj_pos))
		if obj_vel ~= nil then
			obj:setvelocity(calc_velocity(pos, obj_pos,
					obj_vel, radius * 10))
		end
		local damage = (4 / dist) * radius
		obj:set_hp(obj:get_hp() - damage)
	end
end

local function add_effects(pos, radius)
	minetest.add_particlespawner({
		amount = 128,
		time = 1,
		minpos = vector.subtract(pos, radius / 2),
		maxpos = vector.add(pos, radius / 2),
		minvel = {x=-20, y=-20, z=-20},
		maxvel = {x=20,  y=20,  z=20},
		minacc = vector.new(),
		maxacc = vector.new(),
		minexptime = 1,
		maxexptime = 3,
		minsize = 8,
		maxsize = 16,
		texture = "tnt_smoke.png",
	})
end

local function burn(pos)
	local name = minetest.get_node(pos).name
	if name == "tnt:tnt" then
		minetest.sound_play("tnt_ignite", {pos=pos})
		minetest.set_node(pos, {name="tnt:tnt_burning"})
		minetest.get_node_timer(pos):start(1)
	elseif name == "tnt:gunpowder" then
		minetest.sound_play("tnt_gunpowder_burning", {pos=pos, gain=2})
		minetest.set_node(pos, {name="tnt:gunpowder_burning"})
		minetest.get_node_timer(pos):start(1)
	end
end

local function explode(pos, radius)
	local pos = vector.round(pos)
	local vm = VoxelManip()
	local pr = PseudoRandom(os.time())
	local p1 = vector.subtract(pos, radius)
	local p2 = vector.add(pos, radius)
	local minp, maxp = vm:read_from_map(p1, p2)
	local a = VoxelArea:new({MinEdge = minp, MaxEdge = maxp})
	local data = vm:get_data()
	local drops = {}
	local p = {}
	local c_air = minetest.get_content_id("air")
	local c_tnt = minetest.get_content_id("tnt:tnt")
	local c_tnt_burning = minetest.get_content_id("tnt:tnt_burning")
	local c_gunpowder = minetest.get_content_id("tnt:gunpowder")
	local c_gunpowder_burning = minetest.get_content_id("tnt:gunpowder_burning")
	local c_boom = minetest.get_content_id("tnt:boom")
	local c_fire = minetest.get_content_id("fire:basic_flame")
	for z = -radius, radius do
	for y = -radius, radius do
	local vi = a:index(pos.x + (-radius), pos.y + y, pos.z + z)
	for x = -radius, radius do
		if (x * x) + (y * y) + (z * z) <=
				(radius * radius) + pr:next(-radius, radius) then
			local cid = data[vi]
			p.x = pos.x + x
			p.y = pos.y + y
			p.z = pos.z + z
			if cid == c_tnt or cid == c_gunpowder then
				burn(p)
			elseif cid ~= c_tnt_burning and
					cid ~= c_gunpowder_burning and
					cid ~= c_air and
					cid ~= c_fire and
					cid ~= c_boom then
				destroy(drops, p, cid)
			end
		end
		vi = vi + 1
	end
	end
	end
	return drops
end

local function boom(pos)
	minetest.sound_play("tnt_explode", {pos=pos, gain=1.5, max_hear_distance=2*64})
	minetest.set_node(pos, {name="tnt:boom"})
	minetest.get_node_timer(pos):start(0.5)
	local drops = explode(pos, radius)
	entity_physics(pos, radius)
	eject_drops(drops, pos, radius)
	add_effects(pos, radius)
end

minetest.register_node("tnt:tnt", {
	description = "TNT",
	tiles = {"tnt_top.png", "tnt_bottom.png", "tnt_side.png"},
	groups = {dig_immediate=2, mesecon=2},
	sounds = default.node_sound_wood_defaults(),
	on_punch = function(pos, node, puncher)
		if puncher:get_wielded_item():get_name() == "default:torch" then
			minetest.sound_play("tnt_ignite", {pos=pos})
			minetest.set_node(pos, {name="tnt:tnt_burning"})
			minetest.get_node_timer(pos):start(4)
		end
	end,
	mesecons = {effector = {action_on = boom}},
})

minetest.register_node("tnt:tnt_burning", {
	tiles = {
		{
			name = "tnt_top_burning_animated.png",
			animation = {
				type = "vertical_frames",
				aspect_w = 16,
				aspect_h = 16,
				length = 1,
			}
		},
		"tnt_bottom.png", "tnt_side.png"},
	light_source = 5,
	drop = "",
	sounds = default.node_sound_wood_defaults(),
	on_timer = boom,
})

minetest.register_node("tnt:boom", {
	drawtype = "plantlike",
	tiles = {"tnt_boom.png"},
	light_source = default.LIGHT_MAX,
	walkable = false,
	drop = "",
	groups = {dig_immediate=3},
	on_timer = function(pos, elapsed)
		minetest.remove_node(pos)
	end,
})

minetest.register_node("tnt:gunpowder", {
	description = "Gun Powder",
	drawtype = "raillike",
	paramtype = "light",
	sunlight_propagates = true,
	walkable = false,
	tiles = {"tnt_gunpowder.png",},
	inventory_image = "tnt_gunpowder_inventory.png",
	wield_image = "tnt_gunpowder_inventory.png",
	selection_box = {
		type = "fixed",
		fixed = {-1/2, -1/2, -1/2, 1/2, -1/2+1/16, 1/2},
	},
	groups = {dig_immediate=2,attached_node=1},
	sounds = default.node_sound_leaves_defaults(),

	on_punch = function(pos, node, puncher)
		if puncher:get_wielded_item():get_name() == "default:torch" then
			burn(pos)
		end
	end,
})

minetest.register_node("tnt:gunpowder_burning", {
	drawtype = "raillike",
	paramtype = "light",
	sunlight_propagates = true,
	walkable = false,
	light_source = 5,
	tiles = {{
		name = "tnt_gunpowder_burning_animated.png",
		animation = {
			type = "vertical_frames",
			aspect_w = 16,
			aspect_h = 16,
			length = 1,
		}
	}},
	selection_box = {
		type = "fixed",
		fixed = {-1/2, -1/2, -1/2, 1/2, -1/2+1/16, 1/2},
	},
	drop = "",
	groups = {dig_immediate=2,attached_node=1},
	sounds = default.node_sound_leaves_defaults(),
	on_timer = function(pos, elapsed)
		for dx = -1, 1 do
		for dz = -1, 1 do
		for dy = -1, 1 do
			if not (dx == 0 and dz == 0) then
				burn({
					x = pos.x + dx,
					y = pos.y + dy,
					z = pos.z + dz,
				})
			end
		end
		end
		end
		minetest.remove_node(pos)
	end
})

minetest.register_abm({
	nodenames = {"tnt:tnt", "tnt:gunpowder"},
	neighbors = {"fire:basic_flame", "default:lava_source", "default:lava_flowing"},
	interval = 1,
	chance = 1,
	action = burn,
})

minetest.register_craft({
	output = "tnt:gunpowder",
	type = "shapeless",
	recipe = {"default:coal_lump", "default:gravel"}
})

minetest.register_craft({
	output = "tnt:tnt",
	recipe = {
		{"", "group:wood",    ""},
		{"group:wood", "tnt:gunpowder", "group:wood"},
		{"", "group:wood",    ""}
	}
})

-- Fireworks
------------
local FIREWORKS_GROUP = {cracky=3,flammable=3,oddly_breakable_by_hand=1}
local FIREWORKS_SPARK_GROUP = {cracky=3,not_in_creative_inventory=1}

minetest.register_abm({
		nodenames = {"tnt:red", "tnt:blue", "tnt:green", "tnt:purple", "tnt:orange", "tnt:yellow", "tnt:rainbow"},
		interval = 8,
		chance = 1,
		action = function(pos, node, active_object_count, active_object_count_wider)
		if node.name == "tnt:red" then
				minetest.env:remove_node(pos,{name="tnt:red"})
		elseif node.name == "tnt:blue" then
				minetest.env:remove_node(pos,{name="tnt:blue"})
		elseif node.name == "tnt:green" then
				minetest.env:remove_node(pos,{name="tnt:green"})
		elseif node.name == "tnt:purple" then
				minetest.env:remove_node(pos,{name="tnt:purple"})
		elseif node.name == "tnt:orange" then
				minetest.env:remove_node(pos,{name="tnt:orange"})
		elseif node.name == "tnt:yellow" then
				minetest.env:remove_node(pos,{name="tnt:yellow"})
		elseif node.name == "tnt:rainbow" then
				minetest.env:remove_node(pos,{name="tnt:rainbow"})
			end
		end
})

minetest.register_node("tnt:firework_red", {
	description = "Red Fireworks",
	tiles = {"fireworks_firework_red.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:red", {
	drawtype = "plantlike",
	description = "Red",
	tiles = {"fireworks_red.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:firework_blue", {
	description = "Blue Fireworks",
	tiles = {"fireworks_firework_blue.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:blue", {
	drawtype = "plantlike",
	description = "Blue",
	tiles = {"fireworks_blue.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:firework_green", {
	description = "Green Fireworks",
	tiles = {"fireworks_firework_green.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:green", {
	drawtype = "plantlike",
	description = "Green",
	tiles = {"fireworks_green.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:firework_purple", {
	description = "Purple Fireworks",
	tiles = {"fireworks_firework_purple.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:purple", {
	drawtype = "plantlike",
	description = "Purple",
	tiles = {"fireworks_purple.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:firework_orange", {
	description = "Orange Fireworks",
	tiles = {"fireworks_firework_orange.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:orange", {
	drawtype = "plantlike",
	description = "Orange",
	tiles = {"fireworks_orange.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:firework_yellow", {
	description = "Yellow Fireworks",
	tiles = {"fireworks_firework_yellow.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:yellow", {
	drawtype = "plantlike",
	description = "Yellow",
	tiles = {"fireworks_yellow.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:firework_rainbow", {
	description = "Rainbow Fireworks",
	tiles = {"fireworks_firework_rainbow.png"},
	is_ground_content = true,
	groups = FIREWORKS_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_node("tnt:rainbow", {
	drawtype = "plantlike",
	description = "Rainbow",
	tiles = {"fireworks_rainbow.png"},
	light_source = 14,
	walkable = false,
	is_ground_content = true,
	pointable = false,
	groups = FIREWORKS_SPARK_GROUP,
	sounds = default.node_sound_stone_defaults(),
})

minetest.register_craft({
	output = "tnt:firework_red 2",
	recipe = {
		{"default:wood", "default:wood", "default:wood"},
		{"default:wood", "default:torch", "default:wood"},
		{"default:wood", "default:wood", "default:wood"}
	}
})

minetest.register_craft({
	output = "tnt:firework_blue 2",
	recipe = {
		{"tnt:firework_red"}
	}
})

minetest.register_craft({
	output = "tnt:firework_yellow 2",
	recipe = {
		{"tnt:firework_blue"}
	}
})

minetest.register_craft({
	type = "shapeless",
	output = "tnt:firework_green 2",
	recipe = {"tnt:firework_blue", "tnt:firework_yellow"},
})

minetest.register_craft({
	type = "shapeless",
	output = "tnt:firework_purple 2",
	recipe = {"tnt:firework_red", "tnt:firework_blue"},

})

minetest.register_craft({
	type = "shapeless",
	output = "tnt:firework_orange 2",
	recipe = {"tnt:firework_red", "tnt:firework_yellow"},

})

minetest.register_craft({
	type = "shapeless",
	output = "tnt:firework_rainbow 5",
	recipe =
		{"tnt:firework_red", "tnt:firework_blue", "tnt:firework_yellow", "tnt:firework_orange", "tnt:firework_green", "tnt:firework_purple"}

})

function fireworks_activate (pos, node)
local zrand = math.random(-10, 10)
local xrand = math.random(-10,10)
local yrand = math.random(10, 30)
if
	node.name == "tnt:firework_red"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:red'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_red"})
elseif
	node.name == "tnt:firework_blue"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:blue'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_blue"})
elseif
	node.name == "tnt:firework_green"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:green'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_green"})
elseif
	node.name == "tnt:firework_purple"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:purple'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_purple"})
elseif
	node.name == "tnt:firework_orange"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:orange'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_orange"})
elseif
	node.name == "tnt:firework_yellow"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:yellow'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_yellow"})
elseif
	node.name == "tnt:firework_rainbow"
then
local radius = 4
for x=-radius,radius do
for y=-radius,radius do
for z=-radius,radius do
	if x*x+y*y+z*z <= radius*radius then
		minetest.env:add_node({x=pos.x+x+xrand,y=pos.y+y+yrand,z=pos.z+z+zrand},{name='tnt:rainbow'})
end
end
end
end
minetest.env:remove_node(pos,{name="tnt:firework_rainbow"})
end
end
minetest.register_on_punchnode(fireworks_activate)

----- NUKES -------
-- Nuke Mod 1.6 by sfan5 and Licensed under GPLv3

local IRON_TNT_RANGE = 6
local MESE_TNT_RANGE = 12
local HARDCORE_IRON_TNT_RANGE = 4
local HARDCORE_MESE_TNT_RANGE = 4

function spawn_tnt(pos, entname)
	minetest.sound_play("nuke_ignite", {pos = pos,gain = 1.0,max_hear_distance = 8,})
	return minetest.add_entity(pos, entname)
end
function activate_if_tnt(nname, np, tnt_np, tntr)
	if nname == "tnt:iron_tnt" or nname == "tnt:mese_tnt" or nname == "tnt:hardcore_iron_tnt" or nname == "tnt:hardcore_mese_tnt" then
		local e = spawn_tnt(np, nname)
		e:setvelocity({x=(np.x - tnt_np.x)*3+(tntr / 4), y=(np.y - tnt_np.y)*3+(tntr / 3), z=(np.z - tnt_np.z)*3+(tntr / 4)})
	end
end
function do_tnt_physics(tnt_np,tntr)
	local objs = minetest.get_objects_inside_radius(tnt_np, tntr)
	for k, obj in pairs(objs) do
		local v = obj:getvelocity()
		local p = obj:getpos()
		if v ~= nil then
			obj:setvelocity({x=(p.x - tnt_np.x) + (tntr / 4) + v.x, y=(p.y - tnt_np.y) + (tntr / 2) + v.y, z=(p.z - tnt_np.z) + (tntr / 4) + v.z})
		else
			if obj:get_player_name() ~= nil then
				obj:set_hp(obj:get_hp() - 1)
			end
		end
	end
end

-- Iron TNT

minetest.register_craft({
	output = 'tnt:iron_tnt 4',
	recipe = {
		{'','default:wood 1',''},
		{'default:steel_ingot 1','default:coal_lump 1','default:steel_ingot 1'},
		{'','default:wood 1',''}
	}
})
minetest.register_node("tnt:iron_tnt", {
	tile_images = {"nuke_iron_tnt_top.png", "nuke_iron_tnt_bottom.png",
			"nuke_iron_tnt_side.png", "nuke_iron_tnt_side.png",
			"nuke_iron_tnt_side.png", "nuke_iron_tnt_side.png"},
	inventory_image = minetest.inventorycube("nuke_iron_tnt_top.png",
			"nuke_iron_tnt_side.png", "nuke_iron_tnt_side.png"),
	dug_item = '',
	diggable = false,
	description = "Iron TNT",
	mesecons = {
		effector = {
			action_on = function(pos, node)
				minetest.remove_node(pos)
				spawn_tnt(pos, node.name)
				nodeupdate(pos)
			end,
			action_off = function(pos, node) end,
			action_change = function(pos, node) end,
		},
	},
	on_punch = function(pos, node, puncher)
		minetest.remove_node(pos)
		spawn_tnt(pos, node.name)
		nodeupdate(pos)
	end,
})

local IRON_TNT = {
	-- Static definition
	physical = true, -- Collides with things
	-- weight = 5,
	collisionbox = {-0.5,-0.5,-0.5, 0.5,0.5,0.5},
	visual = "cube",
	textures = {"nuke_iron_tnt_top.png", "nuke_iron_tnt_bottom.png",
			"nuke_iron_tnt_side.png", "nuke_iron_tnt_side.png",
			"nuke_iron_tnt_side.png", "nuke_iron_tnt_side.png"},
	-- Initial value for our timer
	timer = 0,
	-- Number of punches required to defuse
	health = 1,
	blinktimer = 0,
	blinkstatus = true,
}

function IRON_TNT:on_activate(staticdata)
	self.object:setvelocity({x=0, y=4, z=0})
	self.object:setacceleration({x=0, y=-10, z=0})
	self.object:settexturemod("^[brighten")
end

function IRON_TNT:on_step(dtime)
	self.timer = self.timer + dtime
	self.blinktimer = self.blinktimer + dtime
	if self.timer>5 then
		self.blinktimer = self.blinktimer + dtime
		if self.timer>8 then
			self.blinktimer = self.blinktimer + dtime
			self.blinktimer = self.blinktimer + dtime
		end
	end
	if self.blinktimer > 0.5 then
		self.blinktimer = self.blinktimer - 0.5
		if self.blinkstatus then
			self.object:settexturemod("")
		else
			self.object:settexturemod("^[brighten")
		end
		self.blinkstatus = not self.blinkstatus
	end
	if self.timer > 10 then
		local pos = self.object:getpos()
		pos.x = math.floor(pos.x+0.5)
		pos.y = math.floor(pos.y+0.5)
		pos.z = math.floor(pos.z+0.5)
		do_tnt_physics(pos, IRON_TNT_RANGE)
		minetest.sound_play("nuke_explode", {pos = pos,gain = 1.0,max_hear_distance = 16,})
		if minetest.get_node(pos).name == "default:water_source" or minetest.get_node(pos).name == "default:water_flowing" then
			-- Cancel the Explosion
			self.object:remove()
			return
		end
		for x=-IRON_TNT_RANGE,IRON_TNT_RANGE do
		for y=-IRON_TNT_RANGE,IRON_TNT_RANGE do
		for z=-IRON_TNT_RANGE,IRON_TNT_RANGE do
			if x*x+y*y+z*z <= IRON_TNT_RANGE * IRON_TNT_RANGE + IRON_TNT_RANGE then
				local np={x=pos.x+x,y=pos.y+y,z=pos.z+z}
				local n = minetest.get_node(np)
				if n.name ~= "air" then
					minetest.remove_node(np)
				end
				activate_if_tnt(n.name, np, pos, IRON_TNT_RANGE)
			end
		end
		end
		end
		self.object:remove()
	end
end

function IRON_TNT:on_punch(hitter)
	self.health = self.health - 1
	if self.health <= 0 then
		self.object:remove()
		hitter:get_inventory():add_item("main", "tnt:iron_tnt")
	end
end

minetest.register_entity("tnt:iron_tnt", IRON_TNT)

-- Mese TNT

minetest.register_craft({
	output = 'tnt:mese_tnt 4',
	recipe = {
		{'','default:wood 1',''},
		{'default:mese_crystal 1','default:coal_lump 1','default:mese_crystal 1'},
		{'','default:wood 1',''}
	}
})
minetest.register_node("tnt:mese_tnt", {
	tile_images = {"nuke_mese_tnt_top.png", "nuke_mese_tnt_bottom.png",
			"nuke_mese_tnt_side.png", "nuke_mese_tnt_side.png",
			"nuke_mese_tnt_side.png", "nuke_mese_tnt_side.png"},
	inventory_image = minetest.inventorycube("nuke_mese_tnt_top.png",
			"nuke_mese_tnt_side.png", "nuke_mese_tnt_side.png"),
	dug_item = '', -- Get nothing
	diggable = false,
	description = "Mese TNT",
	mesecons = {
		effector = {
			action_on = function(pos, node)
				minetest.remove_node(pos)
				spawn_tnt(pos, node.name)
				nodeupdate(pos)
			end,
			action_off = function(pos, node) end,
			action_change = function(pos, node) end,
		},
	},
	on_punch = function(pos, node, puncher)
		minetest.remove_node(pos)
		spawn_tnt(pos, node.name)
		nodeupdate(pos)
	end,
})

local MESE_TNT = {
	-- Static definition
	physical = true, -- Collides with things
	-- weight = 5,
	collisionbox = {-0.5,-0.5,-0.5, 0.5,0.5,0.5},
	visual = "cube",
	textures = {"nuke_mese_tnt_top.png", "nuke_mese_tnt_bottom.png",
			"nuke_mese_tnt_side.png", "nuke_mese_tnt_side.png",
			"nuke_mese_tnt_side.png", "nuke_mese_tnt_side.png"},
	-- Initial value for our timer
	timer = 0,
	-- Number of punches required to defuse
	health = 1,
	blinktimer = 0,
	blinkstatus = true,
}

function MESE_TNT:on_activate(staticdata)
	self.object:setvelocity({x=0, y=4, z=0})
	self.object:setacceleration({x=0, y=-10, z=0})
	self.object:settexturemod("^[brighten")
end

function MESE_TNT:on_step(dtime)
	self.timer = self.timer + dtime
	self.blinktimer = self.blinktimer + dtime
	if self.timer>5 then
		self.blinktimer = self.blinktimer + dtime
		if self.timer>8 then
			self.blinktimer = self.blinktimer + dtime
			self.blinktimer = self.blinktimer + dtime
		end
	end
	if self.blinktimer > 0.5 then
		self.blinktimer = self.blinktimer - 0.5
		if self.blinkstatus then
			self.object:settexturemod("")
		else
			self.object:settexturemod("^[brighten")
		end
		self.blinkstatus = not self.blinkstatus
	end
	if self.timer > 10 then
		local pos = self.object:getpos()
		pos.x = math.floor(pos.x+0.5)
		pos.y = math.floor(pos.y+0.5)
		pos.z = math.floor(pos.z+0.5)
		do_tnt_physics(pos, MESE_TNT_RANGE)
		minetest.sound_play("nuke_explode", {pos = pos,gain = 1.0,max_hear_distance = 16,})
		if minetest.get_node(pos).name == "default:water_source" or minetest.get_node(pos).name == "default:water_flowing" then
			-- Cancel the Explosion
			self.object:remove()
			return
		end
		for x=-MESE_TNT_RANGE,MESE_TNT_RANGE do
		for y=-MESE_TNT_RANGE,MESE_TNT_RANGE do
		for z=-MESE_TNT_RANGE,MESE_TNT_RANGE do
			if x*x+y*y+z*z <= MESE_TNT_RANGE * MESE_TNT_RANGE + MESE_TNT_RANGE then
				local np={x=pos.x+x,y=pos.y+y,z=pos.z+z}
				local n = minetest.get_node(np)
				if n.name ~= "air" then
					minetest.remove_node(np)
				end
				activate_if_tnt(n.name, np, pos, MESE_TNT_RANGE)
			end
		end
		end
		end
		self.object:remove()
	end
end

function MESE_TNT:on_punch(hitter)
	self.health = self.health - 1
	if self.health <= 0 then
		self.object:remove()
		hitter:get_inventory():add_item("main", "tnt:mese_tnt")
	end
end

minetest.register_entity("tnt:mese_tnt", MESE_TNT)

-- Hardcore Iron TNT

minetest.register_craft({
	output = 'tnt:hardcore_iron_tnt 1',
	recipe = {
		{'','default:coal_lump 1',''},
		{'default:coal_lump 1','tnt:iron_tnt 1','default:coal_lump 1'},
		{'','default:coal_lump 1',''}
	}
})
minetest.register_node("tnt:hardcore_iron_tnt", {
	tile_images = {"nuke_iron_tnt_top.png", "nuke_iron_tnt_bottom.png",
			"nuke_hardcore_iron_tnt_side.png", "nuke_hardcore_iron_tnt_side.png",
			"nuke_hardcore_iron_tnt_side.png", "nuke_hardcore_iron_tnt_side.png"},
	inventory_image = minetest.inventorycube("nuke_iron_tnt_top.png",
			"nuke_hardcore_iron_tnt_side.png", "nuke_hardcore_iron_tnt_side.png"),
	dug_item = '', -- Get nothing
	diggable = false,
	description = "Hardcore Iron TNT",
	mesecons = {
		effector = {
			action_on = function(pos, node)
				minetest.remove_node(pos)
				spawn_tnt(pos, node.name)
				nodeupdate(pos)
			end,
			action_off = function(pos, node) end,
			action_change = function(pos, node) end,
		},
	},
	on_punch = function(pos, node, puncher)
		minetest.remove_node(pos)
		spawn_tnt(pos, node.name)
		nodeupdate(pos)
	end,
})

local HARDCORE_IRON_TNT = {
	-- Static definition
	physical = true, -- Collides with things
	-- weight = 5,
	collisionbox = {-0.5,-0.5,-0.5, 0.5,0.5,0.5},
	visual = "cube",
	textures = {"nuke_iron_tnt_top.png", "nuke_iron_tnt_bottom.png",
			"nuke_hardcore_iron_tnt_side.png", "nuke_hardcore_iron_tnt_side.png",
			"nuke_hardcore_iron_tnt_side.png", "nuke_hardcore_iron_tnt_side.png"},
	-- Initial value for our timer
	timer = 0,
	-- Number of punches required to defuse
	health = 1,
	blinktimer = 0,
	blinkstatus = true,
}

function HARDCORE_IRON_TNT:on_activate(staticdata)
	self.object:setvelocity({x=0, y=4, z=0})
	self.object:setacceleration({x=0, y=-10, z=0})
	self.object:settexturemod("^[brighten")
end

function HARDCORE_IRON_TNT:on_step(dtime)
	self.timer = self.timer + dtime
	self.blinktimer = self.blinktimer + dtime
	if self.timer>5 then
		self.blinktimer = self.blinktimer + dtime
		if self.timer>8 then
			self.blinktimer = self.blinktimer + dtime
			self.blinktimer = self.blinktimer + dtime
		end
	end
	if self.blinktimer > 0.5 then
		self.blinktimer = self.blinktimer - 0.5
		if self.blinkstatus then
			self.object:settexturemod("")
		else
			self.object:settexturemod("^[brighten")
		end
		self.blinkstatus = not self.blinkstatus
	end
	if self.timer > 10 then
		local pos = self.object:getpos()
		pos.x = math.floor(pos.x+0.5)
		pos.y = math.floor(pos.y+0.5)
		pos.z = math.floor(pos.z+0.5)
		minetest.sound_play("nuke_explode", {pos = pos,gain = 1.0,max_hear_distance = 16,})
		for x=-HARDCORE_IRON_TNT_RANGE,HARDCORE_IRON_TNT_RANGE do
		for z=-HARDCORE_IRON_TNT_RANGE,HARDCORE_IRON_TNT_RANGE do
			if x*x+z*z <= HARDCORE_IRON_TNT_RANGE * HARDCORE_IRON_TNT_RANGE + HARDCORE_IRON_TNT_RANGE then
				local np={x=pos.x+x,y=pos.y,z=pos.z+z}
				minetest.add_entity(np, "tnt:iron_tnt")
			end
		end
		end
		self.object:remove()
	end
end

function HARDCORE_IRON_TNT:on_punch(hitter)
	self.health = self.health - 1
	if self.health <= 0 then
		self.object:remove()
		hitter:get_inventory():add_item("main", "tnt:hardcore_iron_tnt 1")
	end
end

minetest.register_entity("tnt:hardcore_iron_tnt", HARDCORE_IRON_TNT)

-- Hardcore Mese TNT

minetest.register_craft({
	output = 'tnt:hardcore_mese_tnt 1',
	recipe = {
		{'','default:coal_lump 1',''},
		{'default:coal_lump 1','tnt:mese_tnt 1','default:coal_lump 1'},
		{'','default:coal_lump 1',''}
	}
})
minetest.register_node("tnt:hardcore_mese_tnt", {
	tile_images = {"nuke_mese_tnt_top.png", "nuke_mese_tnt_bottom.png",
			"nuke_hardcore_mese_tnt_side.png", "nuke_hardcore_mese_tnt_side.png",
			"nuke_hardcore_mese_tnt_side.png", "nuke_hardcore_mese_tnt_side.png"},
	inventory_image = minetest.inventorycube("nuke_mese_tnt_top.png",
			"nuke_hardcore_mese_tnt_side.png", "nuke_hardcore_mese_tnt_side.png"),
	dug_item = '', -- Get nothing
	diggable = false,
	description = "Hardcore Mese TNT",
	mesecons = {
		effector = {
			action_on = function(pos, node)
				minetest.remove_node(pos)
				spawn_tnt(pos, node.name)
				nodeupdate(pos)
			end,
			action_off = function(pos, node) end,
			action_change = function(pos, node) end,
		},
	},
	on_punch = function(pos, node, puncher)
		minetest.remove_node(pos)
		spawn_tnt(pos, node.name)
		nodeupdate(pos)
	end,
})

local HARDCORE_MESE_TNT = {
	-- Static definition
	physical = true, -- Collides with things
	-- weight = 5,
	collisionbox = {-0.5,-0.5,-0.5, 0.5,0.5,0.5},
	visual = "cube",
	textures = {"nuke_mese_tnt_top.png", "nuke_mese_tnt_bottom.png",
			"nuke_hardcore_mese_tnt_side.png", "nuke_hardcore_mese_tnt_side.png",
			"nuke_hardcore_mese_tnt_side.png", "nuke_hardcore_mese_tnt_side.png"},
	-- Initial value for our timer
	timer = 0,
	-- Number of punches required to defuse
	health = 1,
	blinktimer = 0,
	blinkstatus = true,
}

function HARDCORE_MESE_TNT:on_activate(staticdata)
	self.object:setvelocity({x=0, y=4, z=0})
	self.object:setacceleration({x=0, y=-10, z=0})
	self.object:settexturemod("^[brighten")
end

function HARDCORE_MESE_TNT:on_step(dtime)
	self.timer = self.timer + dtime
	self.blinktimer = self.blinktimer + dtime
	if self.timer>5 then
		self.blinktimer = self.blinktimer + dtime
		if self.timer>8 then
			self.blinktimer = self.blinktimer + dtime
			self.blinktimer = self.blinktimer + dtime
		end
	end
	if self.blinktimer > 0.5 then
		self.blinktimer = self.blinktimer - 0.5
		if self.blinkstatus then
			self.object:settexturemod("")
		else
			self.object:settexturemod("^[brighten")
		end
		self.blinkstatus = not self.blinkstatus
	end
	if self.timer > 10 then
		local pos = self.object:getpos()
		pos.x = math.floor(pos.x+0.5)
		pos.y = math.floor(pos.y+0.5)
		pos.z = math.floor(pos.z+0.5)
		minetest.sound_play("nuke_explode", {pos = pos,gain = 1.0,max_hear_distance = 16,})
		for x=-HARDCORE_MESE_TNT_RANGE,HARDCORE_MESE_TNT_RANGE do
		for z=-HARDCORE_MESE_TNT_RANGE,HARDCORE_MESE_TNT_RANGE do
			if x*x+z*z <= HARDCORE_MESE_TNT_RANGE * HARDCORE_MESE_TNT_RANGE + HARDCORE_MESE_TNT_RANGE then
				local np={x=pos.x+x,y=pos.y,z=pos.z+z}
				minetest.add_entity(np, "tnt:mese_tnt")
			end
		end
		end
		self.object:remove()
	end
end

function HARDCORE_MESE_TNT:on_punch(hitter)
	self.health = self.health - 1
	if self.health <= 0 then
		self.object:remove()
		hitter:get_inventory():add_item("main", "tnt:hardcore_mese_tnt 1")
	end
end

minetest.register_entity("tnt:hardcore_mese_tnt", HARDCORE_MESE_TNT)
